@@ -18,16 +18,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 module Commands.Ls(lsrecent, lsreplies, lsblocking,
                    lsfollowing, lsfollowers, lsarchive, 
-                   lsrt, lsrtreplies, lsrtarchive, lsdm, lsdmarchive) where
+                   lsrtreplies, status, lsdm, lsdmarchive) where
 import Utils
 import System.Log.Logger
 import Types
 import Text.Printf
 import System.Console.GetOpt
-import Data.List
-import Text.XML.HaXml hiding (when)
 import Download
-import FeedParser
 import Data.ConfigFile
 import Data.String.Utils(strip)
 import Config
@@ -36,8 +33,11 @@ import Control.Monad(when)
 import HSH
 import System.Console.GetOpt.Utils
 import Network.URI
-import Data.Maybe (isJust)
+import Data.Maybe (isJust, isNothing)
 import Network.OAuth.Http.Request
+import Data.Time.Format (formatTime, parseTime)
+import Data.Time.LocalTime (ZonedTime)
+import System.Locale (defaultTimeLocale, rfc822DateFormat)
 
 i = infoM "ls"
 
@@ -50,10 +50,12 @@ stdopts = [Option "a" ["all"] (NoArg ("a", ""))
            Option "l" ["long"] (NoArg ("l", "")) 
                       "Long format output -- more info and \
                       \tab-separated columns",
-           Option "w" ["width"] (ReqArg (stdRequired "w") "WIDTH")
+           widthopt]
+
+widthopt = Option "w" ["width"] (ReqArg (stdRequired "w") "WIDTH")
                       ("Set the margin at which word-wrapping occurs.\n\
                        \Ignored in long format mode. Default is "
-                       ++ show defaultWidth ++ ".")]
+                       ++ show defaultWidth ++ ".")
 
 sinceopts = [
            Option "e" ["exec"] (ReqArg (stdRequired "e") "COMMAND")
@@ -101,8 +103,9 @@ maybeSave section cpath cp args newid =
           isRight _  (Left _)   = False
           isRight v1 (Right v2) = v1 == v2
 
-sinceArgs section cp args =
-    case (lookup "u" args, get cp section "lastid") of
+sinceArgs section cp args maxId = from ++ since where
+    from = maybe [] (\i -> [("max_id", show i)]) maxId
+    since = case (lookup "u" args, get cp section "lastid") of
       (Nothing, _) -> []
       (_, Left _) -> []
       (_, Right a) -> 
@@ -117,15 +120,15 @@ screenNameArgs args =
 -- lsrecent & friends
 --------------------------------------------------
 
-lsrecent = simpleCmd "lsrecent" "List recent updates from those you follow"
+lsrecent = simpleCmd "lsrecent" "List recent updates from your home timeline"
              lsrecent_help
              (stdopts ++ sinceopts) (paginated (statuses_worker "lsrecent"
-                                                "/statuses/friends_timeline"))
+                                                "/statuses/home_timeline"))
 
-lsreplies = simpleCmd "lsreplies" "List recent replies to you"
+lsreplies = simpleCmd "lsreplies" "List recent messages mentioning you"
             lsreplies_help
             (stdopts ++ sinceopts) (paginated (statuses_worker "lsreplies" 
-                                               "/statuses/replies"))
+                                               "/statuses/mentions_timeline"))
 
 lsarchive = simpleCmd "lsarchive" "List recent status updates you posted yourself"
             lsarchive_help
@@ -139,8 +142,9 @@ lsarchive = simpleCmd "lsarchive" "List recent status updates you posted yoursel
                   \those of the given username."
                   ]
 
-
-
+status = simpleCmd "status" "Retrieve a single status"
+         status_help
+         [widthopt] status_worker
 
 lsdm = simpleCmd "lsdm" "List recent direct messages to you"
        lsdm_help
@@ -152,38 +156,45 @@ lsdmarchive = simpleCmd "lsdmarchive" "List recent direct messages you sent"
               (stdopts ++ sinceopts) (paginated (dm_worker "lsdmarchive"
                                                  "/direct_messages/sent"))
 
-lsrt = simpleCmd "lsrt" "List recent retweets from those you follow"
-       lsrt_help
-       (stdopts ++ sinceopts) (paginated (statuses_worker "lsrt"
-                                          "/statuses/retweeted_to_me"))
-       
-lsrtarchive = simpleCmd "lsrtarchive" "List recent retweets you made yourself"
-              lsrtarchive_help
-              (stdopts ++ sinceopts) (paginated (statuses_worker "lsrtarchive"
-                                                 "/statuses/retweeted_by_me"))
-
 lsrtreplies = simpleCmd "lsrtreplies" "List others' retweets of your statuses"
               lsrtreplies_help
               (stdopts ++ sinceopts) (paginated (statuses_worker "lsrtreplies"
                                                  "/statuses/retweets_of_me"))
 
-statuses_worker = generic_worker handleStatus
+statuses_worker = generic_worker handleStatuses
 dm_worker = generic_worker handleDM
 
-generic_worker procfunc section command cpath cp (args, _) page =
-    do xmlstr <- sendAuthRequest GET cp (command ++ ".xml")
-                 (("page", show page) : sinceArgs section cp args
-                  ++ screenNameArgs args)
-                 []
-       debugM section $ "Got doc: " ++ xmlstr
-       results <- procfunc section cp args xmlstr
-       when (page == 1) $
-            maybeSaveList section cpath cp args (map sId results)
-       return results
+status_worker _ cp (args, [statusId]) = do
+  json <- sendAuthRequest GET cp "/statuses/show.json"
+            [("id", statusId)] []
+  debugM "status" $ "Got doc: " ++ show json
+  let TimelineMessage status = decode_json json
+  printStatus "status" cp args status
+status_worker _ _ _ = error "Invalid args to status; see twidge status --help"
+
+
+handleStatuses = handleGeneric (map fromTimeline) printStatus
+
+generic_worker procfunc section command cpath cp (args, _) maxId = do
+  json <- sendAuthRequest GET cp (command ++ ".json")
+            (sinceArgs section cp args maxId
+             ++ screenNameArgs args) []
+  debugM section $ "Got doc: " ++ show json
+  results <- procfunc section cp args json
+  when (isNothing maxId) $
+       maybeSaveList section cpath cp args (map sId results)
+  return (results, nextMaxId results)
+
+nextMaxId :: [Message] -> Maybe Integer
+nextMaxId [] = Nothing
+nextMaxId r = Just ((minimum $ map (read . sId) r) - 1)
+
+status_help =
+ "Usage: twidge status [options] status-id\n"
 
 lsrecent_help =
  "Usage: twidge lsrecent [options]\n\n\
- \You can see the 20 most recent items from your friends with:\n\n\
+ \You can see the 20 most recent items from your timeline with:\n\n\
  \   twidge lsrecent\n\n\
  \To see items that you haven't seen yet, and remember this for the future,\n\
  \use:\n\n\
@@ -194,9 +205,9 @@ lsrecent_help =
 
 lsreplies_help =
  "Usage: twidge lsreplies [options]\n\n\
- \You can see the 20 most recent @replies from others to you with:\n\n\
+ \You can see the 20 most recent @mentions from others of you with:\n\n\
  \   twidge lsreplies\n\n\
- \For more examples, including how to see only unseen replies, please\n\
+ \For more examples, including how to see only unseen mentions, please\n\
  \refer to the examples under twidge lsrecent --help, which also pertain\n\
  \to lsreplies.\n"
 
@@ -208,14 +219,6 @@ lsarchive_help =
  \refer to the examples under twidge lsrecent --help, which also pertain\n\
  \to lsarchive.\n"
 
-lsrt_help = 
- "Usage: twidge lsrt [options]\n\n\
- \You can see the 20 most recent retweets posted by those you follow with:\n\n\
- \   twidge lsrt\n\n\
- \For more examples, including how to see only unseen retweets, please\n\
- \refer to the examples under twidge lsrecent --help, which also pertain\n\
- \to lsreplies.\n"
- 
 lsrtreplies_help = 
  "Usage: twidge lsrtreplies [options]\n\n\
  \You can see the 20 most retweets made of your statuses with:\n\n\
@@ -224,14 +227,6 @@ lsrtreplies_help =
  \refer to the examples under twidge lsrecent --help, which also pertain\n\
  \to lsreplies.\n"
 
-lsrtarchive_help = 
- "Usage: twidge lsrt [options]\n\n\
- \You can see the 20 most recent retweets you made:\n\n\
- \   twidge lsrtarchive\n\n\
- \For more examples, including how to see only unseen retweets, please\n\
- \refer to the examples under twidge lsrecent --help, which also pertain\n\
- \to lsreplies.\n"
-
 
 lsdm_help =
  "Usage: twidge lsdm [options]\n\n\
@@ -249,36 +244,13 @@ lsdmarchive_help =
  \refer to the examples under twidge lsrecent --help, which also pertain\n\
  \to lsdmarchive.\n"
 
-handleStatus = handleGeneric (map procStatuses . getStatuses) printStatus
-handleDM = handleGeneric (map procDM . getDMs) printDM
+handleDM = handleGeneric (map fromDM) printDM
 
-handleGeneric pfunc printfunc section cp args xmlstr = 
-    let doc = getContent . xmlParse section . stripUnicodeBOM $ xmlstr
-        statuses = pfunc doc
+handleGeneric pfunc printfunc section cp args json =
+    let statuses = pfunc $ decode_json json
     in do mapM_ (printfunc section cp args) statuses
           return statuses
 
-procStatuses :: Content -> Message
-procStatuses item = 
-    Message {sId = s (tag "id") item,
-             sSender = s (tag "user" /> tag "screen_name") item,
-             sRecipient = "",
-             sText = unEsc $ s (tag "text") item,
-             sDate = s (tag "created_at") item}
-
-s f item = sanitize $ contentToString (keep /> f /> txt $ item)
-
-procDM :: Content -> Message
-procDM item =
-    Message {sId = s (tag "id") item,
-             sSender = s (tag "sender_screen_name") item,
-             sRecipient = s (tag "recipient_screen_name") item,
-             sText = unEsc $ s (tag "text") item,
-             sDate = s (tag "created_at") item}
-
-getStatuses = tag "statuses" /> tag "status"
-getDMs = tag "direct-messages" /> tag "direct_message"
-
 longStatus :: Message -> String
 longStatus m = printf "%s\t%s\t%s\t%s\t%s\t\n"
                (sId m) (sSender m) (sRecipient m) (sText m) (sDate m)
@@ -315,7 +287,6 @@ printGeneric shortfunc longfunc section cp args m =
                                                             Nothing -> defaultWidth) m)
             (Nothing, Just _) -> putStr (longfunc m)
       Just recipient -> mailto section cp args m recipient
-    where msgid = genMsgId section m cp
 
 mailto section cp args m recipient =
     runIO $ echo (message ++ "\n") -|- (sendmail, ["-t"])
@@ -328,6 +299,10 @@ mailto section cp args m recipient =
                       Right x -> ["From: " ++ (sSender m) ++ " <" ++ x ++ ">",
                                   "Subject: " ++ subject]
                     ) ++ 
+                    (case twitterToRFC822 (sDate m) of
+                      Just d -> [ "Date: " ++ d ]
+                      Nothing -> []
+                    ) ++
                     ["Message-ID: " ++ msgid,
                      "X-Twidge-urlbase: " ++ forceEither (get cp "DEFAULT" "urlbase"),
                      "X-Twidge-server-base: " ++ serverHost cp,
@@ -349,6 +324,11 @@ mailto section cp args m recipient =
                      escapeURIString isUnreserved (sSender m)
                     ,"User home: http://twitter.com/" ++ sSender m
                     ]
+          twitterToRFC822 d =
+            formatTime defaultTimeLocale rfc822DateFormat `fmap` time
+              where
+                time :: Maybe ZonedTime
+                time = parseTime defaultTimeLocale "%a %b %e %H:%M:%S %Z %Y" d
 
 ----------------------------------------------------------------------
 -- Follow/block type commands
@@ -361,7 +341,7 @@ mailto section cp args m recipient =
 lsfollowing = simpleCmd "lsfollowing" "List people you are following"
              lsfollowing_help
              stdopts (paginated lsfollowing_worker)
-lsfollowing_worker = genericfb_worker "lsfollowing" "/statuses/friends"
+lsfollowing_worker = genericfb_worker "lsfollowing" "/friends/list.json"
 lsfollowing_help =
  "Usage: twidge lsfollowing [options] [username]\n\n\
  \If username is given, list the twitter accounts that user is following.\n\
@@ -374,7 +354,7 @@ lsfollowing_help =
 lsfollowers = simpleCmd "lsfollowers" "List people that follow you"
              lsfollowers_help
              stdopts (paginated lsfollowers_worker)
-lsfollowers_worker = genericfb_worker "lsfollowers" "/statuses/followers"
+lsfollowers_worker = genericfb_worker "lsfollowers" "/followers/list.json"
 lsfollowers_help =
  "Usage: twidge lsfollowers [options] [username]\n\n\
  \If username is given, list the twitter accounts that follow the user.\n\
@@ -387,27 +367,27 @@ lsfollowers_help =
 lsblocking = simpleCmd "lsblocking" "List people you are blocking"
              lsblocking_help
              stdopts (paginated lsblocking_worker)
-lsblocking_worker = genericfb_worker "lsblocking" "/blocks/blocking"
+lsblocking_worker = genericfb_worker "lsblocking" "/blocks/list.json"
 lsblocking_help =
  "Usage: twidge lsblocking [options]\n\n\
  \List the twitter accounts that your account is blocking.\n"
 
 ------------------------------------------------------------
 -- Generic follow/block support
--- urlbase should be "/statuses/followers" or "/statuses/friends"
 ------------------------------------------------------------
 
-genericfb_worker cmdname urlbase _ cp (args, user) page =
-    do xmlstr <- sendAuthRequest GET cp url [("page", show page)] []
-       debugM cmdname $ "Got doc: " ++ xmlstr
-       let doc = getContent . xmlParse cmdname . stripUnicodeBOM $ xmlstr
-       let users = map procUsers . getUsers $ doc
+genericfb_worker cmdname url _ cp (args, user) page =
+    do json <- sendAuthRequest GET cp url params []
+       debugM cmdname $ "Got doc: " ++ show json
+       let UserList doc nextPage = decode_json json
+       let users = map fromListedUser doc
        mapM_ (printUser args) users
-       return users
-       
-    where url = case user of
-                  [] -> urlbase ++ ".xml"
-                  [x] -> urlbase ++ "/" ++ x ++ ".xml"
+       return (users, nextPage)
+    where params = pageParams ++ targetParams
+          pageParams = maybe [] (\c -> [("cursor", c)]) page
+          targetParams = case user of
+                  [] -> []
+                  [x] -> [("screen_name", x)]
                   _ -> error $ "Invalid args to " ++ cmdname ++ 
                        "; see twidge " ++ cmdname ++ " --help"
           printUser args (name, userid) = 
@@ -415,13 +395,6 @@ genericfb_worker cmdname urlbase _ cp (args, user) page =
                 Nothing -> putStrLn name
                 Just _ -> printf "%s\t%s\n" userid name
 
-          getUsers = tag "users" /> tag "user"
-
-          procUsers :: Content -> (String, String)
-          procUsers item =
-              (sanitize $ contentToString (keep /> tag "screen_name" /> txt $ item),
-               sanitize $ contentToString (keep /> tag "id" /> txt $ item))
-
 ----------------------------------------------------------------------
 -- Generic Utilities
 ----------------------------------------------------------------------
@@ -435,20 +408,11 @@ Used to wrap around worker functions where multiple pages of data can be
 returned. -}
 paginated workerfunc cppath cp (args, remainder)
     | lookup "a" args == Nothing = 
-        do workerfunc cppath cp (args, remainder) 1
+        do workerfunc cppath cp (args, remainder) Nothing
            return ()
-    | otherwise = paginateloop 1
+    | otherwise = paginateloop Nothing
     where paginateloop page =
-              do r <- workerfunc cppath cp (args, remainder) page
+              do (r, nextPage) <- workerfunc cppath cp (args, remainder) page
                  if null r
                      then return ()
-                     else paginateloop (page + 1)
-
-{- | Twitter has an additional level of escaping for &lt; and &gt; only. 
-Sigh. -}
-unEsc :: String -> String
-unEsc [] = []
-unEsc x 
-  | "&lt;" `isPrefixOf` x = '<' : unEsc (drop 4 x)
-  | "&gt;" `isPrefixOf` x = '>' : unEsc (drop 4 x)
-  | otherwise = head x : unEsc (tail x)
+                     else paginateloop nextPage
